package com.chelvc.cloud.vehicle.server.controller;
import com.chelvc.cloud.vehicle.client.dto.UserReceiveAddressDTO;
import com.chelvc.cloud.vehicle.client.param.UserReceiveAddressModifyParam;
import com.chelvc.cloud.vehicle.server.service.UserReceiveAddressService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * 用户常用地址接口
 *
 * @author liude
 * @date 2024/4/30
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserReceiveAddressController {
    private final UserReceiveAddressService userReceiveAddressService;
    /**
     * 新增用户常用地址
     *
     * @param param 新增参数
     * @return 用户常用地址主键
     */
    @PostMapping("/userReceiveAddress/add")
    public Long add(@SpringQueryMap UserReceiveAddressModifyParam param) {
        return this.userReceiveAddressService.add(param);
    }

    /**
     * 修改用户常用地址
     *
     * @param id    用户常用地址主键
     * @param param 修改参数
     */
    @PutMapping("/userReceiveAddress/update")
    public Long update(@RequestParam("id") Long id,
                       @SpringQueryMap UserReceiveAddressModifyParam param) {
        return this.userReceiveAddressService.update(id,param);
    }

    /**
     * 查询用户常用地址列表
     *
     * @return 用户常用地址信息
     */
    @GetMapping("/userReceiveAddress/list")
    public List<UserReceiveAddressDTO> list() {
        return this.userReceiveAddressService.getList();
    }

    /**
     * 删除用户常用地址
     *
     * @param id    用户常用地址主键
     */
    @PutMapping("/userReceiveAddress/delete")
    public void delete(@RequestParam("id") Long id) {
        this.userReceiveAddressService.delete(id);
    }

}
