package com.chelvc.cloud.vehicle.server.copier;

import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.dto.UserReceiveAddressDTO;
import com.chelvc.cloud.vehicle.client.param.MerchantModifyParam;
import com.chelvc.cloud.vehicle.client.param.UserReceiveAddressModifyParam;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.entity.UserReceiveAddress;
import org.mapstruct.Builder;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.Collection;
import java.util.List;

/**
 * 用户常用地址信息拷贝接口
 *
 * @author liude
 * @date 2024/04/29
 */
@Mapper(builder = @Builder(disableBuilder = true))
public interface UserReceiveAddressCopier {
    /**
     * 对象拷贝接口实例
     */
    UserReceiveAddressCopier INSTANCE = Mappers.getMapper(UserReceiveAddressCopier.class);

    /**
     * 用户常用地址信息拷贝
     *
     * @param userReceiveAddress 用户常用地址信息
     * @return 用户常用地址信息
     */
    UserReceiveAddressDTO copying(UserReceiveAddress userReceiveAddress);

    /**
     * 用户常用地址信息拷贝
     *
     * @param userReceiveAddresses 用户常用地址信息集合
     * @return 用户常用地址信息列表
     */
    List<UserReceiveAddressDTO> copying(Collection<UserReceiveAddress> userReceiveAddresses);

    /**
     * 用户常用地址信息拷贝
     *
     * @param param 用户常用地址更新参数
     * @return 用户常用地址信息
     */
    UserReceiveAddress copying(UserReceiveAddressModifyParam param);

    /**
     * 用户常用地址信息拷贝
     *
     * @param param    用户常用地址更新参数
     * @param userReceiveAddress 用户常用地址信息
     */
    void copying(UserReceiveAddressModifyParam param, @MappingTarget UserReceiveAddress userReceiveAddress);
}
