package com.chelvc.cloud.vehicle.server.service;


import com.chelvc.cloud.vehicle.client.constant.ExamineType;
import com.chelvc.cloud.vehicle.client.dto.DynamicCommentDTO;
import com.chelvc.cloud.vehicle.client.dto.EvaluateDTO;
import com.chelvc.cloud.vehicle.client.param.AddDynamicCommentParam;
import com.chelvc.cloud.vehicle.client.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.client.param.QueryDynamicCommentParam;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;

/**
 * 动态评论Service接口
 *
 * @author igl
 * @date 2023-02-25
 */
public interface DynamicCommentService {

    /**
     * 查询动态评论列表
     */
    Pagination<DynamicCommentDTO> queryPageList(QueryDynamicCommentParam param, Integer pageNum, Integer pageSize);

    /**
     * 修改动态评论
     */
    void insertByBo(AddDynamicCommentParam param, Long userId);

    /**
     * 给评论添加点赞/踩
     * @param dynamicStatus
     * @param contentId
     */
    void addNum(Integer dynamicStatus, Long contentId);

    /**
     * 取消评论点赞/踩
     * @param dynamicStatus
     * @param contentId
     */
    void reduceNum(Integer dynamicStatus, Long contentId);

    /**
     * 评论手动审核
     *
     * @param id
     * @param examine
     * @author igl
     * @date 2023/8/17 17:25
     */
    void examine(Long id, ExamineType examine);

    /**
     * 自动审核
     *
     * @param id 评论标识
     * @param platformStatus 平台状态
     * @param examineResult 审核结果
     * @author igl
     * @date 2023/9/12 17:25
     */
    void autoAudit(Long id, int platformStatus, int examineResult);

    /**
     * 查询评论信息
     */
    DynamicCommentDTO queryById(Long id);

    /**
     * 查看新消息
     * @param userId
     * @return
     */
    Integer queryNewInformNum(Long userId);

    Pagination<EvaluateDTO> selectEvaluateListByUserId(Long userId, Paging paging);

    void changeEvaluateInformFlag(Long userId);

    /**
     * 查询动态信息列表
     */
    Pagination<DynamicCommentDTO> queryAdminPageList(QueryAdminDynamicParam param);

}
