package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.user.client.model.Scope;
import com.chelvc.cloud.vehicle.client.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.*;
import com.chelvc.cloud.vehicle.client.util.DateTimeUtils;
import com.chelvc.cloud.vehicle.server.copier.MerchantCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantMapper;
import com.chelvc.cloud.vehicle.server.dao.RegionMapper;
import com.chelvc.cloud.vehicle.server.entity.Category;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.entity.Region;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.OmsCartItemService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 商家业务操作实现
 *
 * @author Woody
 * @date 2023/5/2
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class MerchantServiceImpl extends ServiceImpl<MerchantMapper, Merchant> implements MerchantService {
    private final GoodsService goodsService;

    private final CategoryService categoryService;

    private final LocationHandler locationHandler;

    private final RegionMapper regionMapper;

    private final OmsCartItemService omsCartItemService;

    private final UserClient userService;

    @Override
    public List<MerchantDTO> listNearbyMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listNearbyMerchants(param, size);
    }

    @Override
    public Pagination<MerchantDTO> indexMerchantPaging(@NonNull LocationQueryPagingParam param) {
        IPage<LocationQueryPagingParam> iPage = new Page<>();
        iPage.setCurrent(param.getPaging().getNumber());
        iPage.setSize(param.getPaging().getSize());
        if (0 == param.getQueryType()){
            Page<MerchantDTO> list = this.baseMapper.listNearbyMerchantPaging(iPage,param);
            return DatabaseContextHolder.pagination(list);
        }else{
            Page<MerchantDTO> page = this.baseMapper.listRecommendMerchantPaging(iPage,param);
            return DatabaseContextHolder.pagination(page);
        }
    }

    @Override
    public List<MerchantDTO> listRecommendMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listRecommendMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listSimpleMerchants(@NonNull MerchantQueryParam param) {
        return this.baseMapper.listSimpleMerchants(param);
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(@NonNull Long id) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                AssertUtils.available(this.getById(id), "商家不存在")
        );

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
//        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(Long id, LocationQueryParam param) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                AssertUtils.available(this.baseMapper.getMerchantById(id, param), "商家不存在")
        );
//        Category category = this.categoryService.getById(merchant.getMainBusiness());
//        if (null != category){
//            merchant.setMainBusinessName(category.getName());
//        }
//        List<Category> categoryList = this.categoryService.getCategories(merchant.getAncillaryBusiness());
//        if (!CollectionUtils.isEmpty(categoryList)){
//            List<String> ancillaryBusinessNames = categoryList.stream()
//                    .map(Category::getName) //
//                    .collect(Collectors.toList());
//            merchant.setAncillaryBusinessName(ancillaryBusinessNames);
//        }
        Long userId = SessionContextHolder.getSession().getId();
        //查询商家是否被收藏
        Integer count = this.baseMapper.merchantFavorite(id, userId);
        if (count > 0) {
            merchant.setFavorites(true);
        }

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(100).build()
        );
        // 获取商家商品分类类型列表
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        if (idSet.isEmpty()) {
            return MerchantDetailDTO.builder().merchant(merchant).goods(goods).build();
        }
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public Long addMerchant(@NonNull MerchantModifyParam param) {
        // 判断商家名称是否存在
        Integer count = this.lambdaQuery()
                .eq(Merchant::getName, param.getName())
                .eq(Merchant::getStatus, MerchantStatus.ONLINE).count();
        AssertUtils.available(count < 1, "商家名称已存在");
        Long userId = SessionContextHolder.getId();
        // 判断商家名称是否存在
        Integer count1 = this.lambdaQuery()
                .eq(Merchant::getUserId, userId).count();
        AssertUtils.available(count < 1, "暂时不支持多商家模式");
        Merchant merchant = MerchantCopier.INSTANCE.copying(param);
        merchant.setUserId(userId);
        merchant.setScore(0D);
        merchant.setSale(0);
        merchant.setStatus(MerchantStatus.ONLINE);
        this.save(merchant);
        List<Long> business = param.getAncillaryBusiness();
        if (StringUtils.notEmpty(param.getMainBusiness())){
            business.add(param.getMainBusiness());
        }
//        this.categoryService.removeByIds(business);
        if (!CollectionUtils.isEmpty(business)){
            List<Category> categories = this.categoryService.getCategories(business);
            if (!CollectionUtils.isEmpty(categories)){
                categories.forEach(item ->{
                    item.setSourceId(item.getId());
                    item.setId(null);
                    item.setMerchantId(merchant.getId());
                    item.setType("1");
                    item.setParentId(0L);
                    this.categoryService.save(item);
                });
                // this.categoryService.saveBatchCategories(categories);
            }
        }

        return merchant.getId();
    }

    @Override
    public void updateMerchant(@NonNull Long id, @NonNull MerchantModifyParam param) {
        Merchant merchant = AssertUtils.available(this.getById(id), "商家不存在");
        MerchantCopier.INSTANCE.copying(param, merchant);
        this.updateById(merchant);
        List<Long> business = param.getAncillaryBusiness();
        if (StringUtils.notEmpty(param.getMainBusiness())){
            business.add(param.getMainBusiness());
        }
//        this.categoryService.removeByIds(business);
        if (!CollectionUtils.isEmpty(business)){
            this.categoryService.deleteByMerchantId(merchant.getId());
            List<Category> categories = this.categoryService.getCategories(business);
            if (!CollectionUtils.isEmpty(categories)){
                categories.forEach(item ->{
                    item.setSourceId(item.getId());
                    item.setId(null);
                    item.setMerchantId(merchant.getId());
                    item.setType("1");
                    item.setParentId(0L);
//                this.categoryService.save(item);
                });
                this.categoryService.saveBatchCategories(categories);
            }
        }
    }

    @Override
    public void deleteMerchant(@NonNull Long id) {
        AssertUtils.available(this.getById(id), "商家不存在");
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<MerchantDTO> getMerchantPaging(@NonNull MerchantPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            param.setMerchantId(merchants.get(0));
        }
        Page<Merchant> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getName()), Merchant::getName, param.getName())
                .eq(Objects.nonNull(param.getRecommend()), Merchant::getRecommend, param.getRecommend())
                .eq(StringUtils.notEmpty(param.getMerchantId()),Merchant::getId,param.getMerchantId())
                .eq(Objects.nonNull(param.getStatus()), Merchant::getStatus, param.getStatus())
                .orderByDesc(Merchant::getCreateTime).page(DatabaseContextHolder.page(param.getPaging()));
        return DatabaseContextHolder.pagination(page, MerchantCopier.INSTANCE::copying);
    }

    @Override
    public MerchantDTO getMerchant(@NonNull Long id) {
        Merchant merchant = AssertUtils.available(this.getById(id), "商家不存在");
        return this.convert(merchant);
    }

    /**
     * 转换商家信息
     *
     * @param merchant 商家实例
     * @return 商家信息
     */
    private MerchantDTO convert(Merchant merchant) {
        return MerchantCopier.INSTANCE.copying(merchant);
    }


    @Override
    public List<Long> getMerchantIdsByUserId(Long userId) {
        List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .list();
        return merchants.stream().map(Merchant::getId).collect(Collectors.toList());
    }

    @Override
    public List<MerchantDTO> getMerchantIdsByUserId() {
        Long userId = SessionContextHolder.getSession().getId();
        List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .list();
        return MerchantCopier.INSTANCE.copying(merchants);
    }

    @Override
    public List<MerchantRankDTO> listRankMerchants(MerchantRankParam param, Long userId) {
        List<MerchantRankDTO> result = new ArrayList<>();
        Integer regionId = param.getRegionId();
        Long categoryId = param.getCategoryId();
        Double longitude = (double) 0;
        Double latitude = (double) 0;
        if (regionId == null) {
            UserDTO user = userService.getUser(userId);
            if (user == null) {
                return result;
            }
            Address address = locationHandler.ip2address(SessionContextHolder.getHost());
            if (address == null) {
                return result;
            }
            regionId = address.getDistrict().getId();
            longitude = address.getLongitude();
            latitude = address.getLatitude();
        }
        Region region = regionMapper.selectById(regionId);
        if (region == null) {
            return result;
        }
        result = baseMapper.listRankMerchants(DateTimeUtils.getMonthStartLocalTime(), categoryId, regionId, longitude, latitude,
                region.getLevel());
        return result;
    }

    @Override
    public List<MerchantRankDTO> listRankMerchants(MerchantRankParam param) {
        List<MerchantRankDTO> result = new ArrayList<>();
        Integer regionId = param.getRegionId();
        Long categoryId = param.getCategoryId();
        Double longitude = (double) 0;
        Double latitude = (double) 0;
        if (regionId == null) {
            Address address = locationHandler.ip2address(SessionContextHolder.getHost());
            if (address == null) {
                return result;
            }
            regionId = address.getDistrict().getId();
            longitude = address.getLongitude();
            latitude = address.getLatitude();
        }
        Region region = regionMapper.selectById(regionId);
        if (region == null) {
            return result;
        }
        result = baseMapper.listRankMerchants(DateTimeUtils.getMonthStartLocalTime(), categoryId, regionId, longitude, latitude,
                region.getLevel());
        return result;
    }

    @Override
    public List<MerchantRankCountDTO> rankMerchants(Integer regionId, Integer limit, Integer level, Long categoryId) {
        return baseMapper.rankMerchants(DateTimeUtils.getLastMonthStartLocalTime(),
                DateTimeUtils.getLastMonthEndLocalTime(), regionId, limit, level, categoryId);
    }

    @Override
    public BigDecimal queryTotalCommissionAmountByLastMonth(Integer regionId, Integer level, Long categoryId) {
        return baseMapper.queryTotalCommissionAmountByLastMonth(DateTimeUtils.getLastMonthStartLocalTime(),
                DateTimeUtils.getLastMonthEndLocalTime(), regionId, level, categoryId);
    }

    @Override
    public MerchantDTO getMerchantById(Long merchantId) {
        return MerchantCopier.INSTANCE.copying(this.getById(merchantId));
    }

    @Override
    public String getMerchantName(Long merchantId) {
        LambdaQueryWrapper<Merchant> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Merchant::getUserId, merchantId);
        Merchant merchant = baseMapper.selectOne(wrapper);
        return merchant != null ? merchant.getName() : "";
    }

    /**
     * 购物车列表
     *
     * @param param
     * @return
     */
    @Override
    public Pagination<MerchantDTO> getUserMineCartPaging(OmsCartItemPagingParam param) {
        Long userId = SessionContextHolder.getSession().getId();
        List<Long> merchants = this.baseMapper.getMerchantIds(userId);
        if (merchants == null && merchants.size() < 1) {
            return null;
        }
        Page<Merchant> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getMerchantName()), Merchant::getName, param.getMerchantName())
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .in(Merchant::getId, merchants)
                .orderByDesc(Merchant::getCreateTime).page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<MerchantDTO> pagination = DatabaseContextHolder.pagination(page, MerchantCopier.INSTANCE::copying);
        pagination.getRecords().forEach(item -> {
            item.setCartItems(this.omsCartItemService.merchantListCartItem(item.getId()));
        });
        return pagination;
    }

    @Override
    public void  updateMerchantBusinessStatus(Long merchantId, String businessStatus) {
        Merchant merchant = AssertUtils.nonnull(this.getById(merchantId), "商家不存在");
        this.baseMapper.updateMerchantBusinessStatus(merchantId, businessStatus, "1");
    }

    /**
     * 每天定时更新商家得营业状态
     */
//    @Scheduled(cron = "5 * * * * ?")
    public void batchMerchantBusinessStatus() {
        List<Merchant> merchantList = this.lambdaQuery().list();
        merchantList.forEach(item -> {
            String opening = "10:00:00,18:00:00";
            String[] openStr = opening.split(",");
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("HH:mm:ss");
            LocalTime time1 = LocalTime.parse(openStr[0], formatter);
            LocalTime time2 = LocalTime.parse(openStr[1], formatter);
            if (time1.isAfter(LocalTime.now()) && time2.isAfter(LocalTime.now())) {
                this.baseMapper.updateMerchantBusinessStatus(item.getId(), "0", "0");
            } else {
                this.baseMapper.updateMerchantBusinessStatus(item.getId(), "1", "0");
            }
        });

    }


    @Override
    public MerchantDTO getMerchantByUserId(Long userId){
        Merchant merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .one();
        return MerchantCopier.INSTANCE.copying(merchants);
    }

    @Override
    public List<MerchantDTO> getMerchantByIds(List<Long> ids) {
        List<MerchantDTO> list = new ArrayList<>();
        for(Long id : ids){
            list.add(getMerchantById(id));
        }
        return list;
    }


    /**
     * 获取商家营业状况
     * @return
     */
    @Override
    public Map<String,Object> operatConditions(){
        Long merchantId = null;
        Map<String,Object> map = new HashMap<>();
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            merchantId = merchants.get(0);
        }
        map = this.baseMapper.operatConditions(merchantId);
        return map;
    }


    /**
     * 经营报表
     * @return 经营报表信息
     */
    @Override
    public List<Map<String,Object>> operatReport(ReportModifyParam param){
        Long merchantId = null;
        List<Map<String,Object>> mapList = new ArrayList<>();
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            merchantId = merchants.get(0);
        }
        if ("0".equals(param.getType())){
            mapList = this.baseMapper.operatReportByYear(merchantId,param.getYear());
            return mapList;
        }else {
            mapList = this.baseMapper.operatReportByMonth(merchantId,param.getYear(),param.getMonth());
            return mapList;
        }
    }

    @Override
    public void merchantAudit(Long id, MerchantStatus status){
        Merchant merchant = AssertUtils.available(this.getById(id), "商家不存在");
        merchant.setStatus(status);
        this.updateById(merchant);
    }

    @Override
    public List<Map<String,Object>> getMerchantReport(ReportModifyParam param){
        if ("0".equals(param.getType())){
            return this.baseMapper.getAddMerchantReportByYear(param.getYear());
        }else {
           return this.baseMapper.getAddMerchantReportByMonth(param.getYear(),param.getMonth());
        }
    }


    @Override
    public Map<String,Object> getMerchantReportCount(ReportModifyParam param){
        Integer month = param.getMonth();
        Integer day = param.getDay();
        if ("0".equals(param.getType())){
            month = null;
            day = null;
        }else if ("1".equals(param.getType())){
            day = null;
        }
        return this.baseMapper.getAddMerchantReport(param.getYear(),month,day);
    }

}
